﻿using ComPDFKit.PDFDocument;
using ComPDFKit.PDFPage;
using ComPDFKitViewer.AnnotEvent;
using ComPDFKitViewer.PdfViewer;
using PDF_Office.Model;
using PDF_Office.ViewModels.Tools;
using Prism.Mvvm;
using Prism.Regions;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;
using System.Windows;
using System.Windows.Media.Imaging;
using Prism.Commands;
using PDF_Office.Helper;
using Microsoft.Office.Interop.Word;
using static Dropbox.Api.Files.FileCategory;
using PDF_Office.Views.PropertyPanel.AnnotPanel;
using System.Windows.Controls;
using PDF_Office.CustomControl;
using System.Windows.Markup;
using ComPDFKitViewer;
using System.Runtime.InteropServices;

namespace PDF_Office.ViewModels.PropertyPanel.AnnotPanel
{
    internal class LinkAnnotPropertyViewModel : BindableBase, INavigationAware
    {
        public AnnotAttribEvent AnnotAttribEvent { get; set; }

        private AnnotPropertyPanel PropertyPanel;
        private AnnotArgsType annotType;

        private LinkAnnotArgs _annotArgs;

        public LinkAnnotArgs AnnotArgs
        {
            get
            {
                return _annotArgs;
            }
            set
            {
                if (_annotArgs != null)
                {
                    _annotArgs.LinkDrawFinished -= _annotArgs_LinkDrawFinished;
                }
                _annotArgs = value;

                if (_annotArgs != null)
                {
                    _annotArgs.LinkDrawFinished += _annotArgs_LinkDrawFinished;
                }
            }
        }

        public AnnotArgsType AnnotType
        {
            get { return annotType; }
            set
            {
                SetProperty(ref annotType, value);
            }
        }

        private string pageNumTextContent;

        public string PageNumTextContent
        {
            get
            {
                //if (string.IsNullOrEmpty(pageNumTextContent))
                //{
                //    return "Enter target page";
                //}
                //if (pageNumTextContent != "Enter target page")
                //{
                if (CheckPageNumVaild(out int pageNum, pageNumTextContent))
                {
                    if (AnnotArgs != null)
                    {
                        AnnotArgs.DestIndex = pageNum - 1;
                        //BtnLocationIsEnabled = true;
                        SetImagePreview(AnnotArgs.DestIndex);
                    }
                }
                else
                {
                    //BtnLocationIsEnabled = false;
                    ImagePreviewVisibility = Visibility.Collapsed;
                    BtnGOorBackVisibility = Visibility.Collapsed;
                }
                //}

                return pageNumTextContent;
            }
            set
            {
                SetProperty(ref pageNumTextContent, value);
            }
        }

        /// <summary>
        /// 设置预文本内容
        /// </summary>
        private string pageNumPlaceHoldText = "Enter target page";

        public string PageNumPlaceHoldText
        {
            get { return pageNumPlaceHoldText; }
            set
            {
                SetProperty(ref pageNumPlaceHoldText, value);
            }
        }

        /// <summary>
        /// 是否显示输入框下方提示文案
        /// </summary>
        private Visibility showPageNumTip = Visibility.Collapsed;

        public Visibility ShowPageNumTip
        {
            get { return showPageNumTip; }
            set
            {
                SetProperty(ref showPageNumTip, value);
            }
        }

        /// <summary>
        /// 输入框下方提示文案
        /// </summary>
        private string pageNumTipText = "Page number error.";

        public string PageNumTipText
        {
            get { return pageNumTipText; }
            set
            {
                SetProperty(ref pageNumTipText, value);
            }
        }

        private bool pageNumTextIsEnabled = false;

        public bool PageNumTextIsEnabled
        {
            get { return pageNumTextIsEnabled; }
            set
            {
                SetProperty(ref pageNumTextIsEnabled, value);
            }
        }

        private bool btnLocationIsEnabled = false;

        public bool BtnLocationIsEnabled
        {
            get { return btnLocationIsEnabled; }
            set
            {
                SetProperty(ref btnLocationIsEnabled, value);
            }
        }

        private WriteableBitmap previewImage;

        public WriteableBitmap PreviewImage
        {
            get { return previewImage; }
            set
            {
                SetProperty(ref previewImage, value);
            }
        }

        private int imagePreviewWidth = 196;

        public int ImagePreviewWidth
        {
            get { return imagePreviewWidth; }
            set
            {
                SetProperty(ref imagePreviewWidth, value);
            }
        }

        private int imagePreviewHeight = 256;

        public int ImagePreviewHeight
        {
            get { return imagePreviewHeight; }
            set
            {
                SetProperty(ref imagePreviewHeight, value);
            }
        }

        private Visibility imagePreviewVisibility = Visibility.Collapsed;

        public Visibility ImagePreviewVisibility
        {
            get { return imagePreviewVisibility; }
            set
            {
                SetProperty(ref imagePreviewVisibility, value);
            }
        }

        private Visibility btnGOorBackVisibility = Visibility.Collapsed;

        public Visibility BtnGOorBackVisibility
        {
            get { return btnGOorBackVisibility; }
            set
            {
                SetProperty(ref btnGOorBackVisibility, value);
            }
        }

        private string btnGOorBackContent = "GO";

        public string BtnGOorBackContent
        {
            get { return btnGOorBackContent; }
            set
            {
                SetProperty(ref btnGOorBackContent, value);
            }
        }

        private bool btnLinkPageIsChecked = true;

        public bool BtnLinkPageIsChecked
        {
            get { return btnLinkPageIsChecked; }
            set
            {
                SetProperty(ref btnLinkPageIsChecked, value);
            }
        }

        private bool btnLinkWebIsChecked = false;

        public bool BtnLinkWebIsChecked
        {
            get { return btnLinkWebIsChecked; }
            set
            {
                SetProperty(ref btnLinkWebIsChecked, value);
            }
        }

        private bool btnLinkMailIsChecked = false;

        public bool BtnLinkMailIsChecked
        {
            get { return btnLinkMailIsChecked; }
            set
            {
                SetProperty(ref btnLinkMailIsChecked, value);
            }
        }

        //private Visibility errorNumTipsVisibility = Visibility.Collapsed;

        //public Visibility ErrorNumTipsVisibility
        //{
        //    get { return errorNumTipsVisibility; }
        //    set
        //    {
        //        SetProperty(ref errorNumTipsVisibility, value);
        //    }
        //}

        //private Visibility errorRangeTipsVisibility = Visibility.Collapsed;

        //public Visibility ErrorRangeTipsVisibility
        //{
        //    get { return errorRangeTipsVisibility; }
        //    set
        //    {
        //        SetProperty(ref errorRangeTipsVisibility, value);
        //    }
        //}
        private int totalPage = 0;

        private bool isLoaded = false;
        private int historyPageIndex = 0;
        private int backPageIndex = 0;
        private CPDFViewer pdfViewer;
        private CPDFDocument document;
        private LinkAnnotProperty linkAnnot;
        public DelegateCommand<object> LoadedCommand { get; set; }
        public DelegateCommand<object> PageNumTextChangedCommand { get; set; }
        public DelegateCommand<object> BtnGOorBackPageCommand { get; set; }
        public DelegateCommand<object> LocationCommand { get; set; }

        //public DelegateCommand<object> ToggleButtonTabCommand { get; set; }

        public LinkAnnotPropertyViewModel()
        {
            LoadedCommand = new DelegateCommand<object>(Loaded);
            PageNumTextChangedCommand = new DelegateCommand<object>(PageNumTextChanged);
            BtnGOorBackPageCommand = new DelegateCommand<object>(BtnGOorBackPageEvent);
            LocationCommand = new DelegateCommand<object>(LocationPage);
            //ToggleButtonTabCommand = new DelegateCommand<object>(ToggleButtonTabSelected);
            isLoaded = true;
        }

        private void LocationPage(object obj)
        {
            if (!string.IsNullOrEmpty(PageNumTextContent))
            {
                AnnotAttribEvent?.UpdateAttrib(AnnotAttrib.LinkType, LINK_TYPE.GOTO);
                AnnotAttribEvent?.UpdateAttrib(AnnotAttrib.LinkDestIndx, Convert.ToInt32(PageNumTextContent) - 1);
                AnnotAttribEvent?.UpdateAnnot();

                if (AnnotArgs != null)
                {
                    AnnotArgs.InvokeLinkSaveCalled(this, EventArgs.Empty);
                }
            }
        }

        private void BtnGOorBackPageEvent(object obj)
        {
            if (obj is object[] arry)
            {
                Button btnGOorBack = arry[0] as Button;
                RadioButton btnLinkPage = arry[1] as RadioButton;
                RadioButton btnLinkWeb = arry[2] as RadioButton;
                RadioButton btnLinkMail = arry[3] as RadioButton;

                if (btnLinkPage != null && btnGOorBack != null && btnLinkWeb != null && btnLinkMail != null)
                {
                    if (btnLinkPage.IsChecked == true)
                    {
                        int pageIndex = Convert.ToInt32(PageNumTextContent) - 1;
                        if (btnGOorBack.Tag.ToString() == "GO")
                        {
                            if (AnnotArgs != null)
                            {
                                historyPageIndex = AnnotArgs.PageIndex;
                            }
                            else
                            {
                                historyPageIndex = backPageIndex;
                            }
                            pdfViewer.GoToPage(pageIndex);

                            ChangeBtnGOorBack("BACK", btnGOorBack);
                            SetImagePreview(historyPageIndex);
                        }
                        else
                        {
                            pdfViewer.GoToPage(historyPageIndex);
                            ChangeBtnGOorBack("GO", btnGOorBack);
                            SetImagePreview(pageIndex);
                        }
                    }
                }
            }
        }

        private void ChangeBtnGOorBack(string flag, Button button)
        {
            switch (flag)
            {
                case "BACK":
                    button.Tag = "BACK";
                    button.Content = "BACK";
                    break;

                case "GO":
                    button.Tag = "GO";
                    button.Content = "GO";
                    break;

                default:
                    break;
            }
        }

        private void PageNumTextChanged(object obj)
        {
            if (obj is TextChangedEventArgs textChangedEventArgs)
            {
                if (textChangedEventArgs.Source is TextBoxWithTip textBox)
                {
                    int pageNum = -1;
                    //if (CheckPageNumVaild(out pageNum))
                    //{
                    //    if (AnnotArgs != null)
                    //    {
                    //        AnnotArgs.DestIndex = pageNum - 1;
                    //        SetImagePreview();
                    //    }
                    //}
                }
            }
        }

        private bool CheckPageNumVaild(out int pageNum, string text)
        {
            pageNum = -1;
            //ErrorNumTipsVisibility = Visibility.Collapsed;
            //ErrorRangeTipsVisibility = Visibility.Collapsed;
            //BtnGOorBackVisibility = Visibility.Collapsed;
            if (string.IsNullOrEmpty(text))
            {
                return false;
            }
            if (text.Trim() != string.Empty)
            {
                if (!int.TryParse(text.Trim(), out pageNum))
                {
                    //ErrorNumTipsVisibility = Visibility.Visible;
                    //ErrorRangeTipsVisibility = Visibility.Collapsed;
                    //textBox.IsError = true;
                    //PageNumTextIsError = true;
                    ShowPageNumTip = Visibility.Visible;
                    PageNumTipText = " Page number error.";
                    return false;
                }
            }
            if (pageNum < 1 || pageNum > totalPage)
            {
                //ErrorNumTipsVisibility = Visibility.Collapsed;
                //ErrorRangeTipsVisibility = Visibility.Visible;
                //textBox.IsError = true;
                //PageNumTextIsError = true;
                ShowPageNumTip = Visibility.Visible;
                PageNumTipText = "  Page number out of range.";
                return false;
            }
            else
            {
                //textBox.IsError = false;
                //PageNumTextIsError = false;
                ShowPageNumTip = Visibility.Collapsed;
            }
            BtnGOorBackVisibility = Visibility.Visible;
            return true;
        }

        private void SetImagePreview(int pageIndex)
        {
            ImagePreviewVisibility = Visibility.Visible;
            BtnGOorBackVisibility = Visibility.Visible;

            int dpiHeight = (int)ImagePreviewHeight;
            int dpiWidth = (int)ImagePreviewWidth;
            if (dpiHeight <= 0 || dpiWidth <= 0)
            {
                return;
            }

            //int pageIndex = AnnotArgs.DestIndex;
            Rect MediaRect = GetPageRect(document, pageIndex, CPDFDisplayBox.MediaBox);

            System.Windows.Size pageSize = GetPageSize(document, pageIndex);
            double scale = Math.Min(dpiWidth / MediaRect.Width, dpiHeight / MediaRect.Height);
            scale = Math.Min(scale, 1);

            int cropWidth = (int)(scale * pageSize.Width);
            int cropHeight = (int)(scale * pageSize.Height);

            byte[] imageDatas = RenderPDFPageToArray(document, pageIndex, new Rect(0, 0, cropWidth, cropHeight), scale, true, true);

            WriteableBitmap WirteBitmap = new WriteableBitmap(cropWidth, cropHeight, 96, 96, PixelFormats.Bgra32, null);
            WirteBitmap.WritePixels(new Int32Rect(0, 0, cropWidth, cropHeight), imageDatas, WirteBitmap.BackBufferStride, 0);

            PreviewImage = WirteBitmap;
        }

        private void _annotArgs_LinkDrawFinished(object sender, bool e)
        {
            //linkAnnot.SetTextBoxEnableOrNot(e, totalPage);
            SetTextBoxEnableOrNot(e, totalPage);
            //AnnotArgs = (LinkAnnotArgs)PropertyPanel.annot;
        }

        private void Loaded(object obj)
        {
            if (obj is CompositeCommandParameter composite)
            {
                if (composite.Parameter is LinkAnnotProperty linkAnnotProperty)
                {
                    linkAnnot = linkAnnotProperty;
                }
            }
        }

        public bool IsNavigationTarget(NavigationContext navigationContext)

        {
            return true;
        }

        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
        }

        public void OnNavigatedTo(NavigationContext navigationContext)
        {
            navigationContext.Parameters.TryGetValue<AnnotPropertyPanel>(ParameterNames.PropertyPanelContentViewModel, out PropertyPanel);
            navigationContext.Parameters.TryGetValue<CPDFViewer>(ParameterNames.PDFViewer, out pdfViewer);

            if (PropertyPanel != null && pdfViewer != null)
            {
                AnnotAttribEvent = PropertyPanel.AnnotEvent;
                AnnotType = PropertyPanel.annot.EventType;

                document = pdfViewer.Document;
                totalPage = pdfViewer.Document.PageCount;

                SetTextBoxEnableOrNot(false, totalPage);

                if (AnnotAttribEvent.Attribs.ContainsKey(AnnotAttrib.LinkType))
                {
                    SetLinkType((LINK_TYPE)AnnotAttribEvent.Attribs[AnnotAttrib.LinkType]);
                }
                if (AnnotAttribEvent.Attribs.ContainsKey(AnnotAttrib.LinkDestIndx) && !AnnotAttribEvent.IsAnnotCreateReset)
                {
                    SetLinkPageNum((int)AnnotAttribEvent.Attribs[AnnotAttrib.LinkDestIndx] + 1);
                    if (AnnotArgs == null)
                    {
                        backPageIndex = pdfViewer.CurrentIndex;
                    }
                    else
                    {
                        AnnotArgs.PageIndex = pdfViewer.CurrentIndex;
                    }
                }

                if (isLoaded)
                {
                    if (pdfViewer != null && pdfViewer.ToolManager != null && pdfViewer.ToolManager.CurrentAnnotArgs?.EventType == AnnotArgsType.AnnotLink)
                    {
                        if (AnnotAttribEvent.IsAnnotCreateReset)
                        {
                            AnnotArgs = pdfViewer.ToolManager.CurrentAnnotArgs as LinkAnnotArgs;
                        }
                    }
                }
                if (AnnotArgs == null)
                {
                    AnnotArgs = (LinkAnnotArgs)PropertyPanel.annot;
                }
            }
        }

        private void SetLinkPageNum(int pageNum)
        {
            if (pageNum > 0 && pageNum <= totalPage)
            {
                PageNumTextContent = pageNum.ToString();
                BtnLocationIsEnabled = true;
                PageNumTextIsEnabled = true;
                ImagePreviewVisibility = Visibility.Visible;
                SetImagePreview(pageNum - 1);
            }
        }

        public void SetLinkType(LINK_TYPE linkType, bool isMail = false)
        {
            BtnLinkPageIsChecked = false;
            BtnLinkMailIsChecked = false;
            BtnLinkWebIsChecked = false;
            switch (linkType)
            {
                case LINK_TYPE.GOTO:
                    BtnLinkPageIsChecked = true;
                    break;

                case LINK_TYPE.URI:
                    if (isMail)
                    {
                        BtnLinkMailIsChecked = true;
                    }
                    else
                    {
                        BtnLinkWebIsChecked = true;
                    }
                    break;

                default:
                    BtnLinkPageIsChecked = true;
                    break;
            }
        }

        private void SetTextBoxEnableOrNot(bool enable, int totalPage)
        {
            this.totalPage = totalPage;

            PageNumTextContent = string.Empty;
            //PageWebText.Text = string.Empty;
            //PageMailText.Text = string.Empty;

            PageNumTextIsEnabled = enable;
            if (PageNumTextIsEnabled == true)
            {
                //defaultPageNumText.Text = string.Format($"1-{totalPage}页");
                //PageNumText.PlaceholderText = string.Format($"1-{totalPage}页");
                PageNumPlaceHoldText = string.Format($"1-{totalPage}页");
            }
            else
            {
                //defaultPageNumText.Text = "Enter target page";
                //PageNumText.PlaceholderText = "Enter target page";
                PageNumPlaceHoldText = "Enter target page";
            }
            ShowPageNumTip = Visibility.Collapsed;
            //BorderPageNumText.Background = enable ? Brushes.White : Brushes.LightGray;

            //PageWebText.IsEnabled = enable;
            //BorderPageWebText.Background = enable ? Brushes.White : Brushes.LightGray;

            //PageMailText.IsEnabled = enable;
            //BorderPageMailText.Background = enable ? Brushes.White : Brushes.LightGray;

            BtnLocationIsEnabled = enable;
            //BtnGOorBack.IsEnabled = false;
        }

        public byte[] RenderPDFPageToArray(CPDFDocument document, int pageIndex, Rect renderRect, double currentZoom, bool renderAnnot = false, bool renderForm = false)
        {
            if (renderRect.Width <= 0 || renderRect.Height <= 0 || document == null)
            {
                return null;
            }
            try
            {
                byte[] bmpData = new byte[(int)renderRect.Width * (int)renderRect.Height * 4];
                currentZoom = currentZoom * 96 / 72;
                int flag = 0;
                if (renderAnnot)
                {
                    flag = 1;
                }
                uint bgColor = 0xFFFFFFFF;
                CPDFPage page = document.PageAtIndex(pageIndex);
                if (page.IsValid())
                {
                    page.RenderPageBitmapWithMatrix((float)currentZoom, renderRect, bgColor, bmpData, flag, renderForm);
                }
                return bmpData;
            }
            catch (Exception ex)
            {
                return null;
            }
        }

        private Rect GetPageRect(CPDFDocument document, int pageIndex, CPDFDisplayBox displayBox)
        {
            CPDFPage currentPage = document.PageAtIndex(pageIndex);
            Rect boundBox = currentPage.GetBoundsForBox(displayBox);

            Rect boundRect = new Rect((int)(boundBox.Left / 72.0 * 96.0), (int)(boundBox.Top / 72.0 * 96.0),
            (int)(boundBox.Width / 72.0 * 96.0), (int)(boundBox.Height / 72.0 * 96.0));

            return boundRect;
        }

        private System.Windows.Size GetPageSize(CPDFDocument document, int pageIndex)
        {
            System.Windows.Size pageSize = document.GetPageSize(pageIndex);
            pageSize.Width = pageSize.Width / 72.0 * 96;
            pageSize.Height = pageSize.Height / 72.0 * 96;

            return pageSize;
        }
    }
}